-- Please consult and update the ADRs in this repo when modifying this schema

-- NOTE This SQL is for initial setup of the database
-- It should not be run if the database already exists

--
-- Name: rbac; Type: SCHEMA; Schema: -; Owner: cc_rbac_api
--

CREATE DATABASE rbac;

CREATE USER cc_rbac;

CREATE SCHEMA rbac;

ALTER SCHEMA rbac OWNER TO cc_rbac;

CREATE USER cc_rbac_api;
GRANT USAGE ON SCHEMA rbac TO cc_rbac_api;
-- Don't grant DELETE, because we only allow soft deletes
GRANT SELECT, INSERT, UPDATE ON ALL TABLES IN SCHEMA rbac TO cc_rbac_api;
-- This lets this user advance with nextval (for inserts in tables using this) but not reset it
GRANT USAGE ON ALL SEQUENCES IN SCHEMA rbac TO cc_rbac_api;
ALTER DEFAULT PRIVILEGES IN SCHEMA rbac GRANT SELECT, INSERT, UPDATE ON TABLES TO cc_rbac_api;
ALTER DEFAULT PRIVILEGES IN SCHEMA rbac GRANT USAGE ON SEQUENCES TO cc_rbac_api;

CREATE USER cc_rbac_ro;
GRANT USAGE ON SCHEMA rbac TO cc_rbac_ro;
GRANT SELECT ON ALL TABLES IN SCHEMA rbac TO cc_rbac_ro;
ALTER DEFAULT PRIVILEGES IN SCHEMA rbac GRANT SELECT ON TABLES TO cc_rbac_ro;


--
-- Name: role_binding; Type: TABLE; Schema: rbac; Owner: -
--

CREATE TABLE rbac.role_binding (
    id text NOT NULL,
    user_id text NOT NULL,
    role_name text NOT NULL,
    organization_id text,
    account_id text,
    cloud_cluster_id text,
    logical_cluster_id text,
    cluster_type text,
    resource_type text,
    resource_name text,
    pattern_type text,
    deleted boolean DEFAULT false NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    created_by text NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL,
    modified_by text NOT NULL,
    last_change_id bigint DEFAULT 1 NOT NULL,
    reason jsonb
);

ALTER TABLE rbac.role_binding OWNER TO cc_rbac;

--
-- Name: role_binding_id_seq; Type: SEQUENCE; Schema: rbac; Owner: -
--

-- This sequence is used by the hashid generator that supplies the id
CREATE SEQUENCE rbac.role_binding_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER TABLE rbac.role_binding_id_seq OWNER TO cc_rbac;

--
-- Name: role_binding_last_change_id_seq; Type: SEQUENCE; Schema: rbac; Owner: -
--

-- This sequence is used to generate a monotonically increasing value that can
-- be used by a Change Data Capture (CDC) process that propagates inserts, updates,
-- and (soft) deletes correctly to the data plane
CREATE SEQUENCE rbac.role_binding_last_change_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER TABLE rbac.role_binding_last_change_id_seq OWNER TO cc_rbac;


--
-- Name: role_binding role_binding_pkey; Type: CONSTRAINT; Schema: rbac; Owner: -
--

ALTER TABLE ONLY rbac.role_binding
    ADD CONSTRAINT role_binding_pkey PRIMARY KEY (id);


--
-- Name: role_binding_last_change_id_idx; Type: INDEX; Schema: rbac; Owner: -
--

CREATE UNIQUE INDEX role_binding_last_change_id_idx ON rbac.role_binding USING btree (last_change_id);


--
-- Name: role_binding_scope_idx; Type: INDEX; Schema: rbac; Owner: -
--

CREATE INDEX role_binding_scope_idx ON rbac.role_binding USING btree (organization_id, account_id, cloud_cluster_id, logical_cluster_id)
    WHERE (deleted = false);


--
-- Name: role_binding_unique_idx; Type: INDEX; Schema: rbac; Owner: -
--

CREATE UNIQUE INDEX role_binding_unique_idx ON rbac.role_binding USING btree (
    user_id,
    role_name,
    COALESCE(organization_id, ''::text),
    COALESCE(account_id, ''::text),
    COALESCE(cloud_cluster_id, ''::text),
    COALESCE(logical_cluster_id, ''::text),
    COALESCE(cluster_type, ''::text),
    COALESCE(resource_type, ''::text),
    COALESCE(resource_name, ''::text),
    COALESCE(pattern_type, ''::text))
    WHERE (deleted = false);


--
-- Name: role_binding_user_idx; Type: INDEX; Schema: rbac; Owner: -
--

CREATE INDEX role_binding_user_idx ON rbac.role_binding USING btree (user_id)
    INCLUDE (deleted);


--
-- Name: update_role_binding(); Type: FUNCTION; Schema: rbac; Owner: -
--

-- Function to enforce that last_change_id and modified are updated
CREATE FUNCTION rbac.update_role_binding() RETURNS trigger
    LANGUAGE plpgsql
    AS $$
BEGIN
    NEW.last_change_id = nextval('rbac."role_binding_last_change_id_seq"');
    NEW.modified = NOW();
    RETURN NEW;
END $$;


--
-- Name: role_binding rbac_update_role_binding_trigger; Type: TRIGGER; Schema: rbac; Owner: -
--

-- Trigger to enforce that last_change_id and modified are updated
-- there are currently no other triggers on this table, but the
-- pg_trigger_depth check prevents recursive loops if some are later added.

CREATE TRIGGER rbac_update_role_binding_trigger
  BEFORE INSERT OR UPDATE ON rbac.role_binding
  FOR EACH ROW
  WHEN ((pg_trigger_depth() < 1))
  EXECUTE FUNCTION rbac.update_role_binding();

INSERT INTO rbac.role_binding (id, user_id, role_name, created_by, modified_by) VALUES ('rb-000000', 'flowserviceadmin', 'CCloudRoleBindingAdmin', 'seed', 'seed');


--
-- Name: extractor_publisher_state; Type: TABLE; Schema: rbac; Owner: -
--

CREATE TABLE rbac.extractor_publisher_state (
    events_kafka_message_sequence_id bigint NOT NULL,
    role_binding_last_change_id bigint NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    publish_type text
);


--
-- Name: extractor_events_kafka_message_sequence_id; Type: SEQUENCE; Schema: rbac; Owner: -
--

CREATE SEQUENCE rbac.extractor_events_kafka_message_sequence_id
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

--
-- Name: extractor_publisher_state extractor_publisher_state_pkey; Type: CONSTRAINT; Schema: rbac; Owner: -
--

ALTER TABLE ONLY rbac.extractor_publisher_state
    ADD CONSTRAINT extractor_publisher_state_pkey PRIMARY KEY (events_kafka_message_sequence_id);

--
-- Name: extractor_events_kafka_log_role_binding_last_change_id_idx; Type: INDEX; Schema: rbac; Owner: -
--

CREATE INDEX extractor_events_kafka_log_role_binding_last_change_id_idx ON rbac.extractor_publisher_state USING btree (role_binding_last_change_id);
